
package org.mule.modules.jobvite.config;

import java.util.List;
import org.apache.commons.lang.StringUtils;
import org.mule.config.spring.MuleHierarchicalBeanDefinitionParserDelegate;
import org.mule.config.spring.util.SpringXMLUtils;
import org.mule.modules.jobvite.processors.UpdateCandidatesMessageProcessor;
import org.mule.util.TemplateParser;
import org.springframework.beans.MutablePropertyValues;
import org.springframework.beans.PropertyValue;
import org.springframework.beans.factory.config.BeanDefinition;
import org.springframework.beans.factory.config.RuntimeBeanReference;
import org.springframework.beans.factory.support.BeanDefinitionBuilder;
import org.springframework.beans.factory.support.ManagedList;
import org.springframework.beans.factory.support.ManagedMap;
import org.springframework.beans.factory.xml.BeanDefinitionParser;
import org.springframework.beans.factory.xml.ParserContext;
import org.springframework.util.xml.DomUtils;
import org.w3c.dom.Element;

public class UpdateCandidatesDefinitionParser
    implements BeanDefinitionParser
{

    /**
     * Mule Pattern Info
     * 
     */
    private TemplateParser.PatternInfo patternInfo;

    public UpdateCandidatesDefinitionParser() {
        patternInfo = TemplateParser.createMuleStyleParser().getStyle();
    }

    public BeanDefinition parse(Element element, ParserContext parserContent) {
        BeanDefinitionBuilder builder = BeanDefinitionBuilder.rootBeanDefinition(UpdateCandidatesMessageProcessor.class.getName());
        String configRef = element.getAttribute("config-ref");
        if ((configRef!= null)&&(!StringUtils.isBlank(configRef))) {
            builder.addPropertyValue("moduleObject", configRef);
        }
        Element candidateUpdatesListElement = null;
        candidateUpdatesListElement = DomUtils.getChildElementByTagName(element, "candidate-updates");
        List<Element> candidateUpdatesMapChilds = null;
        if (candidateUpdatesListElement!= null) {
            String candidateUpdatesRef = candidateUpdatesListElement.getAttribute("ref");
            if ((candidateUpdatesRef!= null)&&(!StringUtils.isBlank(candidateUpdatesRef))) {
                if ((!candidateUpdatesRef.startsWith(patternInfo.getPrefix()))&&(!candidateUpdatesRef.endsWith(patternInfo.getSuffix()))) {
                    builder.addPropertyValue("candidateUpdates", new RuntimeBeanReference(candidateUpdatesRef));
                } else {
                    builder.addPropertyValue("candidateUpdates", candidateUpdatesRef);
                }
            } else {
                ManagedMap candidateUpdates = new ManagedMap();
                candidateUpdatesMapChilds = DomUtils.getChildElementsByTagName(candidateUpdatesListElement, "candidate-update");
                if (candidateUpdatesMapChilds!= null) {
                    if (candidateUpdatesMapChilds.size() == 0) {
                        candidateUpdatesMapChilds = DomUtils.getChildElements(candidateUpdatesListElement);
                    }
                    for (Element candidateUpdatesChild: candidateUpdatesMapChilds) {
                        String candidateUpdatesValueRef = candidateUpdatesChild.getAttribute("value-ref");
                        String candidateUpdatesKeyRef = candidateUpdatesChild.getAttribute("key-ref");
                        Object valueObject = null;
                        Object keyObject = null;
                        if ((candidateUpdatesValueRef!= null)&&(!StringUtils.isBlank(candidateUpdatesValueRef))) {
                            if ((!candidateUpdatesValueRef.startsWith(patternInfo.getPrefix()))&&(!candidateUpdatesValueRef.endsWith(patternInfo.getSuffix()))) {
                                valueObject = new RuntimeBeanReference(candidateUpdatesValueRef);
                            } else {
                                valueObject = candidateUpdatesValueRef;
                            }
                        } else {
                            valueObject = candidateUpdatesChild.getTextContent();
                        }
                        if ((candidateUpdatesKeyRef!= null)&&(!StringUtils.isBlank(candidateUpdatesKeyRef))) {
                            keyObject = new RuntimeBeanReference(candidateUpdatesKeyRef);
                        } else {
                            keyObject = candidateUpdatesChild.getAttribute("key");
                        }
                        if ((keyObject == null)||((keyObject instanceof String)&&StringUtils.isBlank(((String) keyObject)))) {
                            keyObject = candidateUpdatesChild.getTagName();
                        }
                        candidateUpdates.put(keyObject, valueObject);
                    }
                }
                builder.addPropertyValue("candidateUpdates", candidateUpdates);
            }
        }
        BeanDefinition definition = builder.getBeanDefinition();
        definition.setAttribute(MuleHierarchicalBeanDefinitionParserDelegate.MULE_NO_RECURSE, Boolean.TRUE);
        MutablePropertyValues propertyValues = parserContent.getContainingBeanDefinition().getPropertyValues();
        if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.config.spring.factories.PollingMessageSourceFactoryBean")) {
            propertyValues.addPropertyValue("messageProcessor", definition);
        } else {
            if (parserContent.getContainingBeanDefinition().getBeanClassName().equals("org.mule.enricher.MessageEnricher")) {
                propertyValues.addPropertyValue("enrichmentMessageProcessor", definition);
            } else {
                PropertyValue messageProcessors = propertyValues.getPropertyValue("messageProcessors");
                if ((messageProcessors == null)||(messageProcessors.getValue() == null)) {
                    propertyValues.addPropertyValue("messageProcessors", new ManagedList());
                }
                List listMessageProcessors = ((List) propertyValues.getPropertyValue("messageProcessors").getValue());
                listMessageProcessors.add(definition);
            }
        }
        return definition;
    }

    protected String getAttributeValue(Element element, String attributeName) {
        if (!StringUtils.isEmpty(element.getAttribute(attributeName))) {
            return element.getAttribute(attributeName);
        }
        return null;
    }

    private String generateChildBeanName(Element element) {
        String id = SpringXMLUtils.getNameOrId(element);
        if (StringUtils.isBlank(id)) {
            String parentId = SpringXMLUtils.getNameOrId(((Element) element.getParentNode()));
            return ((("."+ parentId)+":")+ element.getLocalName());
        } else {
            return id;
        }
    }

}
